using Blazorise.DataGrid;
using Microsoft.AspNetCore.Components;
using Microsoft.Extensions.Localization;
using Microsoft.Extensions.Logging;
using System.Security.Claims;
using WF_WebAdmin.Model;
using WF_WebAdmin.Service;

namespace WF_WebAdmin.Pages
{
    public partial class ValidQuiz
    {
        private List<Quiz> quizzes;
        [Inject]
        public ILogger<ValidQuiz> Logger { get; set; }

        [Inject]
        public IStringLocalizer<ValidQuiz> Localizer { get; set; }

        [Inject]
        public HttpClient Http { get; set; }

        [Inject]
        public NavigationManager NavigationManager { get; set; }
        
        [Inject]
        public IQuizService QuizService { get; set; }


        /// <summary>
        /// Initializes the component asynchronously by fetching the quizzes that need validation.
        /// This method retrieves a list of quizzes from the `QuizService` that are pending validation when the component is initialized.
        /// </summary>
        protected override async Task OnInitializedAsync()
        {
            // Fetch quizzes that need validation
            quizzes = await QuizService.getQuizzesToValidate();
        }

        /// <summary>
        /// Handles the event when the "Validate" button is clicked for a quiz.
        /// This method calls the `ValidateQuiz` method, passing the specified quiz for validation.
        /// </summary>
        /// <param name="quiz">The quiz that is being validated.</param>
        private void OnValidButton(Quiz quiz)
        {
            // Call the ValidateQuiz method to validate the quiz
            ValidateQuiz(quiz);
        }

        /// <summary>
        /// Validates the specified quiz by setting its `IsValid` property to true and updating its state in the service.
        /// This method logs a message to the console indicating the quiz has been validated, then updates the quiz's validation status.
        /// It then calls the `QuizService.updateQuiz` method to persist the changes.
        /// </summary>
        /// <param name="quiz">The quiz that is being validated.</param>
        private void ValidateQuiz(Quiz quiz)
        {
            // Log the validation action to the console
            LoggerSaveStub.Log(Logger, LogLevel.Information, $"Quiz {quiz.Id} validated!");

            Console.WriteLine($"Quiz {quiz.Id} validated!");

            // Create a new quiz instance (or modify the existing one)
            Quiz newQuiz = quiz;
            newQuiz.IsValid = true;

            // Update the quiz state in the QuizService
            QuizService.updateQuiz(quiz);
        }

        /// <summary>
        /// Handles the event when the "Reject" button is clicked for a quiz.
        /// This method calls the `RejectQuiz` method, passing the specified quiz to be rejected.
        /// </summary>
        /// <param name="quiz">The quiz that is being rejected.</param>
        private void OnRejectButton(Quiz quiz)
        {
            // Call the RejectQuiz method to reject the quiz
            RejectQuiz(quiz);
        }

        /// <summary>
        /// Rejects the specified quiz by logging a rejection message and removing it from the QuizService.
        /// This method logs a message to the console indicating the quiz has been rejected, and then calls the `QuizService.removeQuiz`
        /// method to remove the quiz from the system.
        /// </summary>
        /// <param name="quiz">The quiz that is being rejected.</param>
        private void RejectQuiz(Quiz quiz)
        {
            // Log the rejection action to the console
            LoggerSaveStub.Log(Logger, LogLevel.Information, $"Quiz {quiz.Id} rejected");

            Console.WriteLine($"Quiz {quiz.Id} rejected!");

            // Remove the rejected quiz from the QuizService
            QuizService.removeQuiz(quiz.Id);
        }
    }
}