using System.Text.Json;
using WF_WebAdmin.Model;

namespace WF_WebAdmin.Service;

public class UserServiceStub : IUserService
{
    private readonly string _jsonFilePath = Path.Combine(Environment.CurrentDirectory, "wwwroot", "fake_data_users.json");


    /// <summary>
    /// Asynchronously saves a list of users to a JSON file.
    /// </summary>
    /// <param name="users">The list of <see cref="User"/> objects to be saved to the file.</param>
    /// <returns>A task representing the asynchronous operation.</returns>
    /// <remarks>
    /// This method serializes the provided list of <see cref="User"/> objects into a JSON format using the `JsonSerializer`. 
    /// It then writes the serialized JSON string to the file specified by the `_jsonFilePath`. The JSON is written with indentation for readability.
    /// </remarks>
    public async Task saveUsersJson(List<User> users)
    {
        var json = JsonSerializer.Serialize(users, new JsonSerializerOptions { WriteIndented = true });
        await File.WriteAllTextAsync(_jsonFilePath, json);
    }


    /// <summary>
    /// Asynchronously removes a user from the list of users and saves the updated list to a JSON file.
    /// </summary>
    /// <param name="user">The <see cref="User"/> object to be removed from the list.</param>
    /// <returns>A task representing the asynchronous operation.</returns>
    /// <remarks>
    /// This method retrieves the list of all users using the <see cref="getAllUser"/> method, 
    /// then searches for the specified user by their `Id`. If a matching user is found, 
    /// they are removed from the list, and the updated list is saved back to the JSON file using the <see cref="saveUsersJson"/> method.
    /// </remarks>
    public async Task removeUser(User user)
    {
        var data = await getAllUser();
        var u = data.FirstOrDefault(p => p.Id == user.Id);
        if (u != null)
        {
            data.Remove(u);
            await saveUsersJson(data);
        }
    }


    /// <summary>
    /// Asynchronously updates the role of a user, setting the user as an administrator.
    /// </summary>
    /// <param name="user">The <see cref="User"/> object whose role is to be updated.</param>
    /// <returns>A task representing the asynchronous operation of updating the user's role.</returns>
    /// <remarks>
    /// This method updates the `IsAdmin` property of the specified user to `true`, indicating that the user is an administrator.
    /// It then calls the <see cref="updateUser"/> method to persist the updated user information.
    /// </remarks>
    public Task updateRole(User user)
    {
        user.IsAdmin = true;
        return updateUser(user);
    }


    /// <summary>
    /// Asynchronously downgrades the role of a user, removing their administrator privileges.
    /// </summary>
    /// <param name="user">The <see cref="User"/> object whose role is to be downgraded.</param>
    /// <returns>A task representing the asynchronous operation of downgrading the user's role.</returns>
    /// <remarks>
    /// This method updates the `IsAdmin` property of the specified user to `false`, removing their administrator status.
    /// It then calls the <see cref="updateUser"/> method to persist the updated user information.
    /// </remarks>
    public Task downgradeRole(User user)
    {
        user.IsAdmin = false;
        return updateUser(user);
    }


    /// <summary>
    /// Asynchronously retrieves a list of all users from a JSON file.
    /// </summary>
    /// <returns>A task representing the asynchronous operation, with a result of a list of <see cref="User"/> objects.</returns>
    /// <remarks>
    /// This method checks if the JSON file containing user data exists at the specified file path (`_jsonFilePath`). 
    /// If the file does not exist, it logs a message to the console and returns an empty list of users.
    /// If the file exists, it reads the JSON content, deserializes it into a list of <see cref="User"/> objects, 
    /// and returns that list. If the deserialization results in a null value, an empty list is returned.
    /// </remarks>
    public async Task<List<User>> getAllUser()
    {
        if (!File.Exists(_jsonFilePath))
        {
            Console.Out.WriteLine($"{_jsonFilePath} not found");
            return new List<User>();
        }

        var json = await File.ReadAllTextAsync(_jsonFilePath);
        return JsonSerializer.Deserialize<List<User>>(json) ?? new List<User>();
    }


    /// <summary>
    /// Asynchronously retrieves a paginated list of users from a JSON file.
    /// </summary>
    /// <param name="nb">The number of users to retrieve per page.</param>
    /// <param name="page">The page number for the data to retrieve.</param>
    /// <returns>A task representing the asynchronous operation, with a result of a list of <see cref="User"/> objects.</returns>
    /// <remarks>
    /// This method retrieves all users using the <see cref="getAllUser"/> method, then calculates the range of users to return 
    /// based on the specified page number and the number of users per page (`nb`).
    /// It returns the corresponding subset of users for the given page. If the page exceeds the available number of users, 
    /// it returns the last `nb` users available.
    /// </remarks>
    public async Task<List<User>> getSomeUser(int nb, int page)
    {
        var users = await getAllUser();
        if ((page - 1) * nb + nb > users.Count())
        {
            return users.GetRange(users.Count() - nb, nb);
        }
        return users.GetRange((page - 1) * nb, nb);
    }


    /// <summary>
    /// Asynchronously retrieves a single user by their ID from the JSON file.
    /// </summary>
    /// <param name="id">The ID of the user to retrieve.</param>
    /// <returns>A task representing the asynchronous operation, with a result of the <see cref="User"/> object if found, otherwise null.</returns>
    /// <remarks>
    /// This method retrieves all users using the <see cref="getAllUser"/> method, 
    /// then searches for the user with the specified `id`. If a user with the given ID is found, 
    /// the user is returned. Otherwise, it returns null.
    /// </remarks>
    public async Task<User> getOneUser(int id)
    {
        var data = await getAllUser();
        var u = data.FirstOrDefault(p => p.Id == id);
        if (u != null)
        {
            return u;
        }
        return null;
    }


    public Task<List<User>> reserchUsers(string reserch, List<string> args)
    {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Asynchronously retrieves the total number of users from the JSON file.
    /// </summary>
    /// <returns>A task representing the asynchronous operation, with a result of the total number of users.</returns>
    /// <remarks>
    /// This method retrieves all users using the <see cref="getAllUser"/> method and returns the count of users in the list.
    /// </remarks>
    public async Task<int> getNbUser()
    {
        var data = await getAllUser();
        return data.Count;
    }


    /// <summary>
    /// Asynchronously updates the details of a user in the JSON file.
    /// </summary>
    /// <param name="user">The <see cref="User"/> object containing the updated user details.</param>
    /// <returns>A task representing the asynchronous operation of updating the user.</returns>
    /// <remarks>
    /// This method retrieves all users using the <see cref="getAllUser"/> method, then searches for the user with the specified ID.
    /// If a user with the given ID is found, it updates their details (Name, Email, Image, IsAdmin) based on the provided `user` object.
    /// After updating the user, the modified list of users is saved back to the JSON file using the <see cref="saveUsersJson"/> method.
    /// </remarks>
    public async Task updateUser(User user)
    {
        var data = await getAllUser();
        var person = data.FirstOrDefault(p => p.Id == user.Id);
        if (person != null)
        {
            person.Name = user.Name;
            person.Email = user.Email;
            person.Image = user.Image;
            person.IsAdmin = user.IsAdmin;
            await saveUsersJson(data);
        }
    }
}